#!/usr/bin/env node
var tslib = require('tslib');
var path = require('path');
require('fs');
var arg = require('arg');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var arg__default = /*#__PURE__*/_interopDefaultLegacy(arg);

process.cwd();

function parseCliArgs(argv) {
    var args;
    args = arg__default["default"]({
        "--cwd": String,
        "--output": String,
        "--format": String,
        "--watch": Boolean,
        "--minify": Boolean,
        "--help": Boolean,
        "--version": Boolean,
        "--target": String,
        "--no-sourcemap": Boolean,
        "--external": [String],
        "-h": "--help",
        "-v": "--version",
        "-w": "--watch",
        "-o": "--output",
        "-f": "--format",
        "-m": "--minify",
        "-e": "--external",
    }, {
        permissive: true,
        argv: argv
    });
    var source = args._[0];
    var parsedArgs = {
        source: source,
        format: args["--format"],
        file: args["--output"],
        watch: args["--watch"],
        minify: args["--minify"],
        sourcemap: args["--no-sourcemap"] !== true,
        cwd: args["--cwd"],
        help: args["--help"],
        version: args["--version"],
        target: args["--target"],
        external: args["--external"],
    };
    return parsedArgs;
}
var logger = {
    log: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.log.apply(console, args);
    },
    warn: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.log.apply(console, tslib.__spreadArray(tslib.__spreadArray(["\x1b[33m"], args, false), ["\x1b[0m"], false));
    },
    error: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.error.apply(console, tslib.__spreadArray(tslib.__spreadArray(["\x1b[31m"], args, false), ["\x1b[0m"], false));
    },
};

var version = "2.0.0-alpha.2";

var helpMessage = "\nUsage: bunchee [options]\n\nOptions:\n  -v, --version          output the version number\n  -w, --watch            watch src files changes\n  -m, --minify           compress output. false by default\n  -o, --output <file>    specify output filename\n  -f, --format <format>  specify bundle type: \"esm\", \"cjs\", \"umd\". \"esm\" by default\n  -e, --external <mod>   specify an external dependency\n  --target <target>      build environment, use \"node\" for nodejs runtime\n  --no-sourcemap         disable sourcemap generation, sourcemap generation is enabled by default\n  --cwd <cwd>            specify current working directory\n  -h, --help             output usage information\n";
function help() {
    console.log(helpMessage);
}
function exit(err) {
    logger.error(err);
    process.exit(2);
}
function run(args) {
    return tslib.__awaiter(this, void 0, void 0, function () {
        var source, format, watch, minify, sourcemap, target, cwd, file, outputConfig, entry, bundle, err_1;
        return tslib.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    source = args.source, format = args.format, watch = args.watch, minify = args.minify, sourcemap = args.sourcemap, target = args.target;
                    cwd = args.cwd || process.cwd();
                    file = args.file ? path__default["default"].resolve(cwd, args.file) : args.file;
                    outputConfig = {
                        file: file,
                        format: format,
                        cwd: cwd,
                        target: target,
                        external: args.external || [],
                        watch: !!watch,
                        minify: !!minify,
                        sourcemap: sourcemap === false ? false : true,
                    };
                    if (args.version) {
                        return [2, console.log(version)];
                    }
                    if (args.help) {
                        return [2, help()];
                    }
                    entry = source ? path__default["default"].resolve(cwd, source) : '';
                    bundle = require(".").bundle;
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4, bundle(entry, outputConfig)];
                case 2: return [2, _a.sent()];
                case 3:
                    err_1 = _a.sent();
                    if (err_1.name === 'NOT_EXISTED') {
                        help();
                        return [2, exit(err_1)];
                    }
                    throw err_1;
                case 4: return [2];
            }
        });
    });
}
function main() {
    return tslib.__awaiter(this, void 0, void 0, function () {
        var params, error;
        return tslib.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    try {
                        params = parseCliArgs(process.argv.slice(2));
                    }
                    catch (err) {
                        error = err;
                    }
                    if (error || !params) {
                        if (!error)
                            help();
                        return [2, exit(error)];
                    }
                    return [4, run(params)];
                case 1:
                    _a.sent();
                    return [2];
            }
        });
    });
}
main().catch(exit);
//# sourceMappingURL=cli.js.map
