Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var fs = require('fs');
var path = require('path');
var rollup = require('rollup');
var rollupPluginSwc3 = require('rollup-plugin-swc3');
var commonjs = require('@rollup/plugin-commonjs');
var shebang = require('rollup-plugin-preserve-shebang');
var json = require('@rollup/plugin-json');
var nodeResolve = require('@rollup/plugin-node-resolve');
var rollupPluginTerser = require('rollup-plugin-terser');
require('arg');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var commonjs__default = /*#__PURE__*/_interopDefaultLegacy(commonjs);
var shebang__default = /*#__PURE__*/_interopDefaultLegacy(shebang);
var json__default = /*#__PURE__*/_interopDefaultLegacy(json);
var nodeResolve__default = /*#__PURE__*/_interopDefaultLegacy(nodeResolve);

var rootDir = process.cwd();
var config = {
    rootDir: rootDir,
};

function getPackageMeta() {
    var pkgFilePath = path__default["default"].resolve(config.rootDir, "package.json");
    var targetPackageJson = {};
    try {
        targetPackageJson = JSON.parse(fs__default["default"].readFileSync(pkgFilePath, { encoding: "utf-8" }));
    }
    catch (_) { }
    return targetPackageJson;
}
var logger = {
    log: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.log.apply(console, args);
    },
    warn: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.log.apply(console, tslib.__spreadArray(tslib.__spreadArray(["\x1b[33m"], args, false), ["\x1b[0m"], false));
    },
    error: function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.error.apply(console, tslib.__spreadArray(tslib.__spreadArray(["\x1b[31m"], args, false), ["\x1b[0m"], false));
    },
};

var Module = require("module").Module;
var minifyOptions = {
    compress: true,
    format: {
        comments: 'some',
        wrapFuncArgs: false,
        preserveAnnotations: true,
    },
    mangle: true
};
var hasLoggedTsWarning = false;
function resolveTypescript() {
    var ts;
    var m = new Module("", null);
    m.paths = Module._nodeModulePaths(config.rootDir);
    try {
        ts = m.require("typescript");
    }
    catch (_) {
        if (!hasLoggedTsWarning) {
            hasLoggedTsWarning = true;
            logger.warn("Could not load TypeScript compiler. Try `yarn add --dev typescript`");
        }
    }
    return ts;
}
function getDistPath(distPath) {
    return path.resolve(config.rootDir, distPath);
}
function createInputConfig(entry, pkg, options) {
    var _a;
    var _b;
    var externals = [pkg.peerDependencies, pkg.dependencies, pkg.peerDependenciesMeta]
        .filter(function (n) { return Boolean(n); })
        .map(function (o) { return Object.keys(o); })
        .reduce(function (a, b) { return a.concat(b); }, [])
        .concat(((_b = options.external) !== null && _b !== void 0 ? _b : []).concat(pkg.name ? [pkg.name] : []));
    var useTypescript = options.useTypescript, target = options.target, _c = options.minify, minify = _c === void 0 ? false : _c;
    var typings = pkg.types || pkg.typings;
    var cwd = config.rootDir;
    var plugins = [
        nodeResolve__default["default"]({
            preferBuiltins: target === "node",
            extensions: [".mjs", ".js", ".json", ".node", ".jsx"],
        }),
        commonjs__default["default"]({
            include: /node_modules\//,
        }),
        json__default["default"](),
        shebang__default["default"](),
        useTypescript && require("@rollup/plugin-typescript")(tslib.__assign({ tsconfig: (function () {
                var tsconfig = path.resolve(cwd, "tsconfig.json");
                return fs__default["default"].existsSync(tsconfig) ? tsconfig : undefined;
            })(), typescript: resolveTypescript(), jsx: "react", module: "ES6", target: "ES5", noEmitOnError: !options.watch, sourceMap: options.sourcemap, declaration: !!typings }, (!!typings && {
            declarationDir: path.dirname(path.resolve(cwd, typings)),
        }))),
        useTypescript && minify && rollupPluginTerser.terser({
            compress: {
                "keep_infinity": true,
            },
            format: {
                "comments": /^\s*([@#]__[A-Z]__\s*$|@[a-zA-Z]\s*$)/,
                "wrap_func_args": false,
                "preserve_annotations": true,
            }
        }),
        !useTypescript && rollupPluginSwc3.swc({
            include: /\.(m|c)?[jt]sx?$/,
            exclude: 'node_modules',
            tsconfig: 'tsconfig.json',
            jsc: tslib.__assign({ target: 'es5', loose: true, externalHelpers: false, parser: (_a = {
                        syntax: useTypescript ? 'typescript' : 'ecmascript'
                    },
                    _a[useTypescript ? 'tsx' : 'jsx'] = true,
                    _a.privateMethod = true,
                    _a.classPrivateProperty = true,
                    _a.exportDefaultFrom = true,
                    _a) }, (minify && { minify: minifyOptions })),
            sourceMaps: options.sourcemap,
            inlineSourcesContent: false,
        }),
    ].filter(function (n) { return Boolean(n); });
    return {
        input: entry,
        external: function (id) {
            return externals.some(function (name) { return id === name || id.startsWith(name + "/"); });
        },
        plugins: plugins,
        treeshake: {
            propertyReadSideEffects: false,
        },
        onwarn: function (warning, warn) {
            if (warning.code &&
                ['MIXED_EXPORTS', 'PREFER_NAMED_EXPORTS'].includes(warning.code))
                return;
            warn(warning);
        },
    };
}
function createOutputOptions(options, pkg) {
    var format = options.format, useTypescript = options.useTypescript;
    var tsCompilerOptions = {};
    if (useTypescript) {
        var ts = resolveTypescript();
        var tsconfigPath = path.resolve(config.rootDir, "tsconfig.json");
        if (fs__default["default"].existsSync(tsconfigPath)) {
            var tsconfigJSON = ts.readConfigFile(tsconfigPath, ts.sys.readFile).config;
            tsCompilerOptions = ts.parseJsonConfigFileContent(tsconfigJSON, ts.sys, "./").options;
        }
    }
    var exportPaths = getExportPaths(pkg);
    var useEsModuleMark = Boolean(tsCompilerOptions.esModuleInterop ||
        (exportPaths.main &&
            exportPaths.module));
    var file = path.resolve(options.file);
    return {
        name: pkg.name,
        dir: path.dirname(file),
        entryFileNames: path.basename(file),
        format: format,
        exports: "named",
        esModule: useEsModuleMark && format !== "umd",
        freeze: false,
        strict: false,
        sourcemap: options.sourcemap,
    };
}
function findExport(field) {
    if (!field)
        return;
    if (typeof field === "string")
        return field;
    var value = field["."] || field["import"] || field["module"] || field["default"];
    return findExport(value);
}
function parseExport(exportsCondition) {
    var paths = {};
    if (typeof exportsCondition === "string") {
        paths.export = exportsCondition;
    }
    else {
        paths.main = paths.main || exportsCondition["require"] || exportsCondition["node"] || exportsCondition["default"];
        paths.module = paths.module || exportsCondition["module"];
        paths.export = findExport(exportsCondition);
    }
    return paths;
}
function getExportPaths(pkg) {
    var pathsMap = {};
    var mainExport = {};
    if (pkg.main) {
        mainExport.main = pkg.main;
    }
    if (pkg.module) {
        mainExport.module = pkg.module;
    }
    pathsMap['.'] = mainExport;
    var exportsConditions = pkg.exports;
    if (exportsConditions) {
        if (typeof exportsConditions === 'string') {
            mainExport.export = exportsConditions;
        }
        else {
            var exportKeys = Object.keys(exportsConditions);
            if (exportKeys.some(function (key) { return key.startsWith('.'); })) {
                exportKeys.forEach(function (subExport) {
                    pathsMap[subExport] = parseExport(exportsConditions[subExport]);
                });
            }
            else {
                Object.assign(mainExport, parseExport(exportsConditions));
            }
        }
    }
    pathsMap['.'] = mainExport;
    return pathsMap;
}
function getExportDist(pkg) {
    var paths = getExportPaths(pkg)['.'];
    var dist = [];
    if (paths.main) {
        dist.push({ format: "cjs", file: getDistPath(paths.main) });
    }
    if (paths.module) {
        dist.push({ format: "esm", file: getDistPath(paths.module) });
    }
    if (paths.export) {
        dist.push({ format: "esm", file: getDistPath(paths.export) });
    }
    if (dist.length === 0) {
        dist.push({ format: "cjs", file: getDistPath("dist/index.js") });
    }
    return dist;
}
function getSubExportDist(pkg, subExport) {
    var pkgExports = pkg.exports || {};
    var dist = [];
    if (typeof pkgExports === 'string') {
        dist.push({ format: pkg.type === "module" ? "esm" : "cjs", file: getDistPath(pkgExports) });
    }
    else {
        var exports_1 = pkgExports[subExport];
        if (subExport.endsWith(".json"))
            return dist;
        if (typeof exports_1 === 'string') {
            dist.push({ format: "esm", file: getDistPath(exports_1) });
        }
        else {
            if (exports_1.require) {
                dist.push({ format: "cjs", file: getDistPath(exports_1.require) });
            }
            if (exports_1.import) {
                dist.push({ format: "esm", file: getDistPath(exports_1.import) });
            }
        }
    }
    return dist;
}
function createRollupConfig(entry, pkg, cliArgs, entryExport) {
    var file = cliArgs.file, format = cliArgs.format;
    var ext = path.extname(entry);
    var useTypescript = ext === ".ts" || ext === ".tsx";
    var options = tslib.__assign(tslib.__assign({}, cliArgs), { useTypescript: useTypescript });
    var inputOptions = createInputConfig(entry, pkg, options);
    var outputExports = entryExport
        ? getSubExportDist(pkg, entryExport)
        : getExportDist(pkg);
    var outputConfigs = outputExports
        .map(function (exportDist) {
        return createOutputOptions(tslib.__assign(tslib.__assign({}, cliArgs), { file: exportDist.file, format: exportDist.format, useTypescript: useTypescript }), pkg);
    });
    if (file) {
        outputConfigs = [
            createOutputOptions(tslib.__assign(tslib.__assign({}, cliArgs), { file: file, format: format, useTypescript: useTypescript }), pkg)
        ];
    }
    return {
        input: inputOptions,
        output: outputConfigs,
    };
}

function assignDefault(options, name, defaultValue) {
    if (!(name in options) || options[name] == null) {
        options[name] = defaultValue;
    }
}
function bundle(entryPath, _a) {
    if (_a === void 0) { _a = {}; }
    var cwd = _a.cwd, options = tslib.__rest(_a, ["cwd"]);
    config.rootDir = path.resolve(process.cwd(), cwd || "");
    assignDefault(options, "format", "es");
    assignDefault(options, "sourcemap", true);
    if (options.format === "esm") {
        options.format = "es";
    }
    var npmPackage = getPackageMeta();
    var _b = npmPackage.bunchee || {}, entries = _b.entry, customConfig = tslib.__rest(_b, ["entry"]);
    var isSingleEntry = typeof entries === 'string';
    var hasMultiEntries = entries
        && !isSingleEntry
        && Object.keys(entries).length > 0;
    if (isSingleEntry) {
        entryPath = path.resolve(config.rootDir, entries);
    }
    if (!fs__default["default"].existsSync(entryPath)) {
        var hasEntryFile = entryPath === '' ? '' : fs__default["default"].statSync(entryPath).isFile();
        if (!hasEntryFile && !hasMultiEntries) {
            var err = new Error("Entry file is not existed");
            err.name = "NOT_EXISTED";
            return Promise.reject(err);
        }
        if (hasMultiEntries) {
            Object.assign(options, customConfig);
            var rollupConfigs = Object.keys(entries).map(function (entryExport) {
                var source = entries[entryExport];
                var rollupConfig = createRollupConfig(path.resolve(cwd, source), npmPackage, options, entryExport);
                return rollupConfig;
            });
            return Promise.all(rollupConfigs.map(function (rollupConfig) {
                return runBundle(rollupConfig);
            }));
        }
    }
    Object.assign(options, customConfig);
    var rollupConfig = createRollupConfig(entryPath, npmPackage, options);
    if (options.watch) {
        return Promise.resolve(runWatch(rollupConfig));
    }
    return runBundle(rollupConfig);
}
function runWatch(_a) {
    var input = _a.input, output = _a.output;
    var watchOptions = [tslib.__assign(tslib.__assign({}, input), { output: output, watch: {
                exclude: ["node_modules/**"],
            } })];
    var watcher = rollup.watch(watchOptions);
    watcher.on('event', function (event) {
        if (event.code === 'ERROR') {
            onError(event.error);
        }
    });
    return watcher;
}
function runBundle(_a) {
    var input = _a.input, output = _a.output;
    return rollup.rollup(input).then(function (bundle) {
        var writeJobs = output.map(function (options) {
            return bundle.write(options);
        });
        return Promise.all(writeJobs);
    }, onError);
}
function onError(error) {
    if (!error)
        return;
    if (error.frame) {
        process.stdout.write(error.frame + "\n");
    }
    if (error.stack) {
        process.stdout.write(error.stack + "\n");
    }
    throw error;
}

exports.bundle = bundle;
//# sourceMappingURL=index.js.map
